@@ -4,32 +4,42 @@
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-""" Thoroughly document Bokeh enumerations
+''' Thoroughly document Bokeh enumerations
 
-The ``bokeh-enum`` directive generates useful type information
-for the enumeration, including all the allowable values. If the
-number of values is large, the full list is put in a collapsible
-code block.
+The ``bokeh-enum`` directive generates useful documentation for enumerations,
+including all the allowable values. If the number of values is large, the full
+list is put in a collapsible code block.
 
-This directive takes the name of a Bokeh enum variable and the
-module to find the value as an argument::
+This directive takes the name of a Bokeh enum variable as the argument and the
+module name as an option. An optional description may be added as content:
+
+.. code-block:: rest
 
     .. bokeh-enum:: baz
         :module: bokeh.sphinxext.sample
 
+        Specify a baz style
+
 Examples
 --------
 
-For the following definition of ``bokeh.sphinxext.sample.Bar``::
-
-    baz = enumeration("a", "b", "c")
-
-the above usage yields the output:
+The directive above will generate the following output:
 
     .. bokeh-enum:: baz
         :module: bokeh.sphinxext.sample
 
-"""
+        Specify a baz style
+
+Although ``bokeh-enum`` may be used explicitly, it is more often convenient in
+conjunction with the :ref:`bokeh.sphinxext.bokeh_autodoc` extension. Together,
+the same output above will be generated directly from the following code:
+
+.. code-block:: python
+
+    #: Specify a baz style
+    baz = enumeration("a", "b", "c")
+
+'''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
@@ -49,7 +59,6 @@ import textwrap
 
 # External imports
 from docutils.parsers.rst.directives import unchanged
-
 from sphinx.errors import SphinxError
 
 # Bokeh imports
@@ -63,7 +72,6 @@ from .templates import ENUM_DETAIL
 __all__ = (
     'BokehEnumDirective',
     'setup',
-    'wrapper',
 )
 
 #-----------------------------------------------------------------------------
@@ -74,21 +82,16 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-wrapper = textwrap.TextWrapper(subsequent_indent='    ')
-
 class BokehEnumDirective(BokehDirective):
 
     has_content = True
     required_arguments = 1
-    optional_arguments = 2
-
     option_spec = {
         'module': unchanged,
         'noindex': lambda x: True, # directives.flag weirdly returns None
     }
 
     def run(self):
-
         name = self.arguments[0]
 
         try:
@@ -101,7 +104,7 @@ class BokehEnumDirective(BokehDirective):
         fullrepr = repr(enum)
         if len(fullrepr) > 180:
             shortrepr = fullrepr[:40] + " .... " + fullrepr[-40:]
-            fullrepr = wrapper.wrap(fullrepr)
+            fullrepr = _wrapper.wrap(fullrepr)
         else:
             shortrepr = fullrepr
             fullrepr = None
@@ -118,12 +121,15 @@ class BokehEnumDirective(BokehDirective):
         return self._parse(rst_text, "<bokeh-enum>")
 
 def setup(app):
+    ''' Required Sphinx extension setup function. '''
     app.add_directive_to_domain('py', 'bokeh-enum', BokehEnumDirective)
 
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
 
+_wrapper = textwrap.TextWrapper(subsequent_indent='    ')
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
